#   ---------------------------------------------------------------------------------
#   Copyright (c) Microsoft Corporation. All rights reserved.
#   Licensed under the MIT License. See LICENSE in project root for information.
#   ---------------------------------------------------------------------------------
"""This is a Sample Python file."""


from __future__ import annotations
import re
import datetime
from typing import Union

def _roundtimestamp(dt:datetime.datetime, target) -> datetime.datetime:
    """
    Rounds a timestamp to the day
    """
    if target == 'd':
        return dt.replace(hour=0, minute=0, second=0, microsecond=0)
    elif target == 'M':
        return dt.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
    elif target == 'Y':
        return dt.replace(day=1, month=1, hour=0, minute=0, second=0, microsecond=0)


def _timedelta(value, unit) -> datetime.timedelta:
    """
    Returns the timedelta object for the given value and unit
    """

    if unit == "d":
        return datetime.timedelta(days=int(value))
    elif unit == "W":
        return datetime.timedelta(weeks=int(value))
    elif unit == "M":
        return datetime.timedelta(weeks=int(value) * 4.34524)
    elif unit == "Y":
        return datetime.timedelta(weeks=int(value) * 52.1775)
    else:
        raise ValueError(f"Invalid unit '{unit}'")


def parse_shorthand_datetime(datestr:str) -> Union[datetime.datetime, None]:

    if 'now' in datestr:
        # Relative datetime string in relation to current day
        datestr = datestr.replace(' ','') # Remove linebreaks
        value = re.findall("[-+]?[.]?[\d]+(?:,\d\d\d)*[\.]?\d*(?:[eE][-+]?\d+)?", datestr)
        if not value:
            value = [0]

        unit = re.findall("[dWMY]", datestr)

        dt = datetime.datetime.now() + _timedelta(value[0], unit[0])

        if '/' in datestr:
            return _roundtimestamp(dt, unit[-1])
        else:
            return dt

    else:
        return None






import pytest
import shorthand_datetime

import numpy as np

import datetime

FAKE_TIME = datetime.datetime(2024, 11, 15, 17, 5, 55)

@pytest.fixture
def patch_datetime_now(monkeypatch):

    class mydatetime(datetime.datetime):
        @classmethod
        def now(cls):
            return FAKE_TIME

    monkeypatch.setattr(datetime, 'datetime', mydatetime)

# Pytest will discover and run all test functions named `test_*` or `*_test`.

def test_version():
    """ check sdypy_template_project exposes a version attribute """
    assert hasattr(shorthand_datetime, "__version__")
    assert isinstance(shorthand_datetime.__version__, str)

def test_patch_datetime(patch_datetime_now):
    assert datetime.datetime.now() == FAKE_TIME

def test_shorthand(patch_datetime_now):
    from shorthand_datetime.shorthand import parse_shorthand_datetime

    s = 'now-7d'
    dt_7d = parse_shorthand_datetime(s)
    assert dt_7d.strftime('%Y%m%d') == '20241108'

    s = 'now-1M'
    dt_M = parse_shorthand_datetime(s)
    assert dt_M.strftime('%Y%m%d') == '20241016'

    s = 'now-1Y'
    dt_y = parse_shorthand_datetime(s)
    assert dt_y.strftime('%Y%m%d') == '20231116'

def test_rounded_shorthand(patch_datetime_now):
    from shorthand_datetime.shorthand import parse_shorthand_datetime

    s = 'now/d'
    dt_d = parse_shorthand_datetime(s)
    assert dt_d.second == 0
    assert dt_d.strftime('%Y%m%d_%H%M%S') == '20241115_000000'

    s = 'now/M'
    dt_7d = parse_shorthand_datetime(s)
    assert dt_7d.strftime('%Y%m%d') == '20241101'

    s = 'now/Y'
    dt_7d = parse_shorthand_datetime(s)
    assert dt_7d.strftime('%Y%m%d') == '20240101'

    s = 'now-2Y/Y'
    dt_7d = parse_shorthand_datetime(s)
    assert dt_7d.strftime('%Y%m%d') == '20220101'

    s = 'now-1M/M'
    dt_7d = parse_shorthand_datetime(s)
    assert dt_7d.strftime('%Y%m%d') == '20241001'






import os
import re
from setuptools import setup

base_path = os.path.dirname(__file__)

# Read the project version from "__init__.py"
regexp = re.compile(r'.*__version__ = [\'\"](.*?)[\'\"]', re.S)

init_file = os.path.join(base_path, 'shorthand_datetime', '__init__.py')
with open(init_file, 'r') as f:
    module_content = f.read()

    match = regexp.match(module_content)
    if match:
        version = match.group(1)
    else:
        raise RuntimeError(
            'Cannot find __version__ in {}'.format(init_file))

# Read the "README.rst" for project description
with open('README.rst', 'r') as f:
    readme = f.read()


if __name__ == '__main__':
    setup(
        name='shorthand_datetime',
        description='Parse shorthand datetime strings inspired by Grafana',
        long_description=readme,
        license='MIT license',
        version=version,
        author='Wout Weijtjens',
        author_email='wout.weijtjens@gmail.com',
        maintainer='Wout Weijtjens',
        maintainer_email='wout.weijtjens@gmail.com',
        keywords=[],
        packages=['shorthand_datetime'],
        classifiers=['Development Status :: 3 - Alpha',
                     'Intended Audience :: Developers',
                     'Programming Language :: Python :: 3.9']
    )
